@@ -1,5 +1,5 @@
 /*
- * $Id: md5.h,v 1.2 2003/10/20 18:59:57 tomcollins Exp $
+ * $Id: md5.h,v 1.7 2007/05/22 03:58:59 rwidmer Exp $
  * Copyright (C) 2000-2003 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or
@@ -19,11 +19,7 @@
 #ifndef VPOPMAIL_MD5_H
 #define VPOPMAIL_MD5_H
 
-#ifdef __alpha
-typedef unsigned int uint32;
-#else
-typedef unsigned long uint32;
-#endif
+#include "cdb/uint32.h"
 
 struct MD5Context {
 	uint32 buf[4];
