@@ -1,6 +1,6 @@
 /*
- * $Id: vchkpw.c,v 1.11 2004/01/13 15:59:42 tomcollins Exp $
- * Copyright (C) 1999-2003 Inter7 Internet Technologies, Inc.
+ * $Id: vchkpw.c,v 1.21 2007/10/01 22:31:06 rwidmer Exp $
+ * Copyright (C) 1999-2004 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -92,12 +92,20 @@ void read_user_pass();
 void vlog(int verror, char *TheUser, char *TheDomain, char *ThePass, char *TheName, char *IpAddr, char *LogLine);
 void vchkpw_exit(int err);
 void run_command(char *prog);
-int authcram(unsigned char *response, unsigned char *challenge, unsigned char *password);
-int authapop(unsigned char *password, unsigned char *timestamp, unsigned char *clearpass);
+int authcram( char *response, char *challenge, char *password);
+int authapop( char *password, char *timestamp, char *clearpass);
 
 #define POP_CONN  0
 #define SMTP_CONN 1
 #define IMAP_CONN 2
+#define WEBMAIL_CONN 3
+
+/* POP/IMAP connections from the following IPs will be classified as
+ * "web mail" instead of POP/IMAP.  On single-server networks, this
+ * will typically be just 'localhost'.  For clusters, add the IP
+ * addresses of all webmail servers.
+ */
+char *webmailips[] = { "127.0.0.1" };
 
 int ConnType = 0;
 
@@ -106,7 +114,7 @@ int main( int argc, char **argv)
  char *tmpstr;
 
   if ( (IpAddr = get_remote_ip())  == NULL) IpAddr="";
-  if ( (tmpstr = getenv("TCPLOCALPORT")) == NULL) LocalPort = 110;
+  if ( (tmpstr = getenv("TCPLOCALPORT")) == NULL) LocalPort = 0;
   else LocalPort = atoi(tmpstr);
 
   /* Check which port they are coming in on and 
@@ -161,6 +169,16 @@ int main( int argc, char **argv)
       break;
   }
 
+  if ((ConnType == IMAP_CONN) || (ConnType == POP_CONN)) {
+    int i;
+    for (i = 0; i < (sizeof(webmailips)/sizeof(webmailips[0])); i++) {
+      if (strcmp (IpAddr, webmailips[i]) == 0) {
+        strcpy(VchkpwLogName, "vchkpw-webmail");
+        ConnType = WEBMAIL_CONN;
+        break;
+      }
+    }
+  }
 
   /* read in the user name and password from file descriptor 3 */
   read_user_pass();
@@ -390,6 +408,9 @@ void login_virtual_user()
   int apopaccepted = -1;
   int cramaccepted = -1;
   char AuthType[15] = "PLAIN";
+#ifdef MIN_LOGIN_INTERVAL
+  time_t last_time;
+#endif
 
   /* If thier directory path is empty make them a new one */
   if ( vpw->pw_dir == NULL || vpw->pw_dir[0]==0 ) {
@@ -414,18 +435,21 @@ void login_virtual_user()
     }
   }
 
-#ifdef CLEAR_PASS 
-  /* Check CRAM-MD5 auth */
-  if(ConnType == SMTP_CONN) {
-	  /* printf("vchkpw: smtp auth\n"); */
-    cramaccepted = authcram(ThePass,TheChallenge,vpw->pw_clear_passwd);
-    if(cramaccepted == 0) strcpy(AuthType, "CRAM-MD5");
-  }
+#ifdef CLEAR_PASS
+  /* only check for one-way hashed passwords if we have a valid cleartext */
+  if ((vpw->pw_clear_passwd != NULL && vpw->pw_clear_passwd[0] != 0)) {
+    /* Check CRAM-MD5 auth */
+    if(ConnType == SMTP_CONN) {
+      /* printf("vchkpw: smtp auth\n"); */
+      cramaccepted = authcram(ThePass,TheChallenge,vpw->pw_clear_passwd);
+      if(cramaccepted == 0) strcpy(AuthType, "CRAM-MD5");
+    }
 
-  /* Check APOP auth */
-  if(ConnType == POP_CONN) {
-    apopaccepted = authapop(ThePass,TheChallenge,vpw->pw_clear_passwd);
-    if(apopaccepted == 0) strcpy(AuthType, "APOP");
+    /* Check APOP auth */
+    if(ConnType == POP_CONN) {
+      apopaccepted = authapop(ThePass,TheChallenge,vpw->pw_clear_passwd);
+      if(apopaccepted == 0) strcpy(AuthType, "APOP");
+    }
   }
 #endif
 
@@ -503,6 +527,15 @@ void login_virtual_user()
     vchkpw_exit(1);
   }
 
+    /* Check if they are allowed webmail access
+    */
+  else if ( ConnType == WEBMAIL_CONN && (vpw->pw_flags & NO_WEBMAIL)) {
+    snprintf(LogLine, sizeof(LogLine), "%s: webmail access denied %s@%s:%s",
+             VchkpwLogName, TheUser, TheDomain, IpAddr);
+    vlog(VLOG_ERROR_ACCESS, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine);
+    vchkpw_exit(1);
+  }
+
     /* Check if they are allowed imap access
     */
   else if ( ConnType == IMAP_CONN && (vpw->pw_flags & NO_IMAP)) {
@@ -523,8 +556,35 @@ void login_virtual_user()
   /* If authentication logging is enabled
    * update the authentication time on the account
    */
+
+  /*  NOTE:  Need to extend this to handle 
+   *  grace count.  Each time a login is
+   *  attempted up to grace count it should
+   *  be allowed, and the grace counter 
+   *  incremented.  Once the number of attempts
+   *  exceeds the grace value then start to 
+   *  deny logins.  This allows someone to
+   *  check their email extra often for a short
+   *  time, if they are expecting an important
+   *  message without penalty, but if they just
+   *  set the pop3 login interval below the 
+   *  min interval eventually logins will be
+   *  denied.
+   *
+   *  the grace count limit is already stored in
+   *  MIN_LOGIN_GRACE
+   */
+
 #ifdef ENABLE_AUTH_LOGGING
+#ifdef MIN_LOGIN_INTERVAL
+  last_time = vget_lastauth(vpw, TheDomain );
+#endif
   vset_lastauth(TheUser,TheDomain,IpAddr);
+#ifdef MIN_LOGIN_INTERVAL
+  if(( vget_lastauth(vpw,TheDomain ) - last_time ) < MIN_LOGIN_INTERVAL ) { 
+    vchkpw_exit(1);
+  }
+#endif
 #endif
 
 #ifdef POP_AUTH_OPEN_RELAY
@@ -645,43 +705,46 @@ void vlog(int verror, char *TheUser, char *TheDomain, char *ThePass,
     syslog(LOG_NOTICE, sysc(LogLine));
   }
 
-#ifdef ENABLE_MYSQL_LOGGING
+#ifdef ENABLE_SQL_LOGGING
   /* always log to mysql if mysql logging is enabled and it 
    * is not internal error 
    */
 
   if ( (verror == VLOG_ERROR_PASSWD) && ( ENABLE_LOGGING==1 || ENABLE_LOGGING==2 || ENABLE_LOGGING==3 || ENABLE_LOGGING==4 ) ) {
-      if ( (logmysql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-          syslog(LOG_NOTICE,"vchkpw: can't write MySQL logs");
+      if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
+          syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
+      }
+      if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
+          syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
       }
   } else if ( verror == VLOG_ERROR_INTERNAL ) {
-      if ( (logmysql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-        syslog(LOG_NOTICE,"vchkpw: can't write MySQL logs");
+      if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
+        syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
       }
   } else if ( verror == VLOG_ERROR_LOGON ) {
-      if ( (logmysql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-        syslog(LOG_NOTICE,"vchkpw: can't write MySQL logs");
+      if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
+        syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
       }
   } else if ( verror == VLOG_ERROR_ACCESS ) {
-      if ( (logmysql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-        syslog(LOG_NOTICE,"vchkpw: can't write MySQL logs");
+      if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
+        syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
       }
   } else if ( verror == VLOG_AUTH && ( ENABLE_LOGGING == 1 || ENABLE_LOGGING == 4 ) ) {
-      if ( (logmysql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
-        syslog(LOG_NOTICE,"vchkpw: can't write MySQL logs");
+      if ( (logsql(verror, TheUser, TheDomain, ThePass, TheName, IpAddr, LogLine) ) != 0 ) {
+        syslog(LOG_NOTICE,"vchkpw: can't write SQL logs");
       }
   }
 #endif
 }
 
-int authcram(unsigned char *response, unsigned char *challenge, unsigned char *password)
+int authcram( char *response, char *challenge, char *password)
 {
    unsigned char digest[16];
-   unsigned char digascii[33];
+   char digascii[33];
    unsigned char h;
    int j;
 
-   hmac_md5( challenge, strlen(challenge), password, strlen(password), digest);
+   hmac_md5( (unsigned char *) challenge, strlen(challenge), (unsigned char *) password, strlen(password), digest);
 
    digascii[32]=0;
    
@@ -696,28 +759,37 @@ int authcram(unsigned char *response, unsigned char *challenge, unsigned char *p
    return(strcmp(digascii,response));
 }
 
-int authapop(unsigned char *password, unsigned char *timestamp, unsigned char *clearpass)
+int authapop( char *password, char *timestamp, char *clearpass)
 {
-#ifdef USE_ACTIVE_DIR
-  return(-1);
-#else
   MD5_CTX context;
   unsigned char digest[16];
   char encrypted[16*2+1];
   char *s;
   int i;
- 
+
+  unsigned char *utimestamp;
+  unsigned char *uclearpass;
+
+  utimestamp = malloc( strlen( timestamp ));
+  memcpy( utimestamp, timestamp, strlen( timestamp ));
+
+  uclearpass = malloc( strlen( clearpass ));
+  memcpy( uclearpass, clearpass, strlen( clearpass ));
+
   MD5Init(&context);
-  MD5Update(&context, timestamp, strlen(timestamp));
-  MD5Update(&context, clearpass, strlen(clearpass));
+  MD5Update(&context, utimestamp, strlen(timestamp));
+  MD5Update(&context, uclearpass, strlen(clearpass));
   MD5Final(digest, &context);
+
+  free( utimestamp );
+  free( uclearpass );
+
   s = encrypted;
-  for (i = 0; i < sizeof(digest); ++i) {
+  for (i = 0; i < (int)sizeof(digest); ++i) {
     *s = hextab[digest[i]/16]; ++s;
     *s = hextab[digest[i]%16]; ++s;
   }
   *s = '\0';
  
   return strcmp(password,encrypted);
-#endif
 }
