@@ -1,5 +1,10 @@
+/* This program, intended as a Maildir++-aware replacement for qmail-local
+ * has not been maintained and may never have worked properly.  It
+ * should not be used.
+ */
+
 /*
- * $Id: vqmaillocal.c,v 1.3 2003/12/17 03:39:50 tomcollins Exp $
+ * $Id: vqmaillocal.c,v 1.9 2007/05/22 03:59:02 rwidmer Exp $
  * Copyright (C) 2002 Inter7 Internet Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -667,7 +672,7 @@ void run_command(char *prog)
      exit(-1);
   }
 
-  wait(&wstat);
+  waitpid(child,&wstat,0);
 
 }
 
